import java.util.*;

public class Spring extends PhysicsElement {
   private static int id=0;  // Spring identification
   protected final double restLength;
   private final double stiffness;
   protected Ball a_end, b_end;

   private Spring(){   // nobody can create a spring without state
      this(0,0);
   }
   public Spring(double restLength, double stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public void attachEnd (Ball sa) {  // note: we attach a spring to a ball, 
      if(a_end==null){                             //       not the other way around.
         a_end = sa;
         sa.attachSpring(this);
      }
      else {
        if(b_end==null){
          b_end = sa;
          sa.attachSpring(this);
       }
      }
   }
   private double getStrech() {
      if ((a_end == null) || (b_end == null))
         return 0;
      double ax=a_end.getPosition();
      double bx=b_end.getPosition();
      if (ax<bx)
        return (bx-ax-restLength);
      else
        return (ax-bx-restLength);
   }
   public double getForce(Ball ball) {
      double force = 0;
      if ((ball != a_end) && (ball != b_end)){
         return force;
      }
      else{
          force = stiffness*( restLength - Math.abs( b_end.getPosition() - a_end.getPosition() ) );
          if (ball == b_end){
              return (force);
          }
          else{
              return (-force);
          }
      }
   }
   
   public void computeNextState(double delta_t){
   }
   public void updateState(){
   }

   public String getDescription() {
     //  to be coded by you
      String ss = "Spring_"+this.getId() +" "+"a_end\tb_end";
      return ss;
   }
   public String getState() {
     //  to be coded by you
      double ax=a_end.getPosition();
      double bx=b_end.getPosition();
      String ss = ax + "\t" + bx;
      return ss;
   }
}