import java.util.*;

public class Ball extends PhysicsElement implements Attachable{
   private static int id=0;  // Ball identification number
   private final double mass;
   private double pos_t;     // current position at time t
   private double pos_tPlusDelta;  // next position in delta time in future
   private double speed_t;   // speed at time t
   private double speed_tPlusDelta;   // speed in delta time in future
   private double a_t;    // acceleration at time t
   private double a_tMinusDelta; 
   private double total_force = 0;
   private Attacher stuck0 = null ;
   private Attacher stuck1 = null ;
   
   private Ball(){   // nobody can create a ball without state
     this(1.0,0,0);
   }
   public Ball(double mass, double position, double speed){
      super(id++);
      this.mass = mass;
      pos_t = position;
      speed_t = speed;
   }
   
	public double getPosition(){
		return pos_t;
	}
	public double getAceleration(){
		return a_t;
	}
    public double getSpeed(){
        return speed_t;
    }
	public void attach(Attacher st){
		if (stuck0 == null)
			stuck0 = st;
		else if (stuck1 == null)
			stuck1 = st;
	}
	private double getNetForce(){
		total_force = 0;
		if(stuck0!=null)
			total_force += stuck0.getForce(this);
		if(stuck1!=null)
			total_force += stuck1.getForce(this);
		return total_force;
	}

    public void computeNextState(double delta_t) {
		a_tMinusDelta = a_t;
		if(stuck0!=null && stuck1!=null)
			a_t = this.getNetForce()/mass;
		pos_tPlusDelta = pos_t + speed_t*delta_t + (4*a_t-a_tMinusDelta)*(delta_t*delta_t)/6;
		speed_tPlusDelta = speed_t + (3*a_t-a_tMinusDelta)*delta_t/2;
	}

	public void updateState(){
		//a_t se actualiza en metodo anterior
		pos_t = pos_tPlusDelta;
		speed_t = speed_tPlusDelta;
	}

	public String getDescription() {
		return "Ball_"+this.getId()+":x";
	}
	public String getState() {
		return Double.toString(pos_t);
	}
}
	