public class Dashpot extends PhysicsElement implements Attacher
{
	
   private static int id=0;  // Dashpot identification
   private final double viscosity;
   protected Attachable a_end, b_end;

   private Dashpot(){   // nobody can create a dashpot without state
      this(0);
   }
   public Dashpot(double viscosity){
      super(id++);
      this.viscosity = viscosity;
      a_end = b_end = null;
   }
   public void attachEnd (Attachable sa) {  // note: we attach a dashpot to a ball,
      if(a_end==null){ //       not the other way around.
			 a_end = sa;
			 a_end.attach(this);
		  }           
	  else if(b_end==null)
		 {
			 b_end = sa;
			 b_end.attach(this);
		 }
   }
   
    
    private double getSpeed() {
        if ((a_end == null) || (b_end == null))
            return 0;
        double ax=a_end.getSpeed();
        double bx=b_end.getSpeed();
        if (ax<bx)
            return (bx-ax);
        else
            return (ax-bx);
    }

    
   public double getForce(Attachable attached) {
      double force = 0;
      if (( attached != a_end) && (attached != b_end))
        return force;
      if (a_end==attached){
        //verificamos direccion de fuerza para conexion izquierda
        if (a_end.getAceleration()>0){
          return force = this.getSpeed()*viscosity*(-1);
        }
        else if (a_end.getAceleration()<0){
          return force = this.getSpeed()*viscosity;
        }
      }
      else{
        //verificamos direccion de fuerza para conexion derecha
        if (b_end.getAceleration()>0){
          return force = this.getSpeed()*viscosity*(-1);
        }
        else if (b_end.getAceleration()<0){
          return force = this.getSpeed()*viscosity;
        }
      }
	   return force;
   }
    
   public void computeNextState(double delta_t){
   } 
   public void updateState(){
   }

   public String getDescription() {
     return "Dashpot_"+this.getId()+":a_end\tb_end";
   }
    
   public String getState() {
     double a,b;
  	 a=b=0;
  	 if(a_end!=null)
  	 {
  		 a=a_end.getPosition();
  	 }
  	 if(b_end!=null)
  	 {
  		 b=b_end.getPosition();
  	 }
  	 return Double.toString(a)+"\t"+Double.toString(b);
   }
}







