public class Spring extends PhysicsElement implements Attacher{
   private static int id=0;  // Spring identification
   protected final double restLength;
   private final double stiffness;
   protected Attachable a_end, b_end;

   private Spring(){   // nobody can create a spring without state
      this(0,0);
   }
   public Spring(double restLength, double stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public void attachEnd (Attachable sa) {  // note: we attach a spring to a ball, 
      if(a_end==null) //       not the other way around.
         {
			 a_end = sa;
			 a_end.attach(this);
		 }
	  else if(b_end==null)
		 {
			 b_end = sa;
			 b_end.attach(this);
		 }
   }
   private double getStrech() {
      if ((a_end == null) || (b_end == null))
         return 0;
      double ax=a_end.getPosition();
      double bx=b_end.getPosition();
      if (ax<bx)
        return (bx-ax-restLength);
      else
        return (ax-bx-restLength);
   }
   public double getForce(Attachable attached) {
      double force = 0;
      if (( attached != a_end) && (attached != b_end))
         return force;
      
	  if(a_end==attached)
	  {
		  if(a_end.getPosition() < b_end.getPosition())
		  {
			  return this.getStrech()*stiffness;
		  }
		  else
		  {
			  return this.getStrech()*stiffness*(-1);
		  }
	  }
	  else
	   {
		   if(a_end.getPosition() < b_end.getPosition())
		   {
			   return this.getStrech()*stiffness*(-1);
		   }
		   else
		   {
			   return this.getStrech()*stiffness;
		   }
	   }
   }
   public void computeNextState(double delta_t){
   } 
   public void updateState(){
   }

   public String getDescription() {
     return "Spring_"+this.getId()+":a_end\tb_end";
   }
   public String getState() {
     double a,b;
	 a=b=0;
	 if(a_end!=null)
	 {
		 a=a_end.getPosition();
	 }
	 if(b_end!=null)
	 {
		 b=b_end.getPosition();
	 }
	 return Double.toString(a)+"\t"+Double.toString(b);
   }
}
