import java.util.*;

public class Spring extends PhysicsElement {
   private static int id=0;  // Spring identification
   protected final double restLength;
   private final double stiffness;
   protected Attachable a_end, b_end;

   private Spring(){   // nobody can create a spring without state
      this(0,0);
   }
   public Spring(double restLength, double stiffness){
      super(id++);
      this.restLength = restLength;
      this.stiffness = stiffness;
      a_end = b_end = null;
   }
   public void attachEnd (Attachable sa) {
      if(a_end==null){
         a_end = sa;
         sa.attach(this);
      }
      else {
        if(b_end==null){
          b_end = sa;
          sa.attach(this);
        }
      }
   }
   private double getStrech() {
      if ((a_end == null) || (b_end == null))
         return 0;
      double ax=a_end.getPosition();
      double bx=b_end.getPosition();
      if (ax<bx)
        return (bx-ax-restLength);
      else
        return (ax-bx-restLength);
   }
   public double getForce(Attachable att) {
      double force = 0;
      if ((att != a_end) && (att != b_end))
        return force;
      if (a_end==att){
        if(a_end.getPosition()<b_end.getPosition()){
          return this.getStrech()*stiffness;
        }
        else{
          return -(this.getStrech()*stiffness);
        }
      }
      else{
        if(a_end.getPosition()<b_end.getPosition()){
          return -(this.getStrech()*stiffness);
        }
        else{
          return this.getStrech()*stiffness;
        }
      }
   }
   
   public void computeNextState(double delta_t){
   }
   public void updateState(){
   }

   public String getDescription() {
     //  to be coded by you
      String ss = "Spring_"+this.getId() +" "+"a_end\tb_end";
      return ss;
   }
   public String getState() {
     //  to be coded by you
      double ax=a_end.getPosition();
      double bx=b_end.getPosition();
      String ss = ax + "\t" + bx;
      return ss;
   }
}